"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _maxSafeInteger = _interopRequireDefault(require("@babel/runtime/core-js/number/max-safe-integer"));

var _sample2 = _interopRequireDefault(require("lodash/sample"));

var _isFunction2 = _interopRequireDefault(require("lodash/isFunction"));

var _omit2 = _interopRequireDefault(require("lodash/omit"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _isString2 = _interopRequireDefault(require("lodash/isString"));

var _ava = _interopRequireDefault(require("ava"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _tree = _interopRequireDefault(require("./tree"));

const UUIDV4REGEX = /^[0-9A-F]{8}-[0-9A-F]{4}-[4][0-9A-F]{3}-[89AB][0-9A-F]{3}-[0-9A-F]{12}$/i;
/**
 * These tests are abstracted so that they can be used in multiple store scenarios.
 * For example it was originally developed with an in-memory sqlit3 db for speedy development.
 * Later it can easily test a PG or MYSQL store without any changes to the tests themselves.
 * The tests simply expect the same interface.
 */

var _default = async store => {
  const tree = (0, _tree.default)((await store));
  (0, _ava.default)('A node can generate child "factories"', async t => {
    t.plan(76);
    const factories = await tree.generateFactory(15);
    t.is(factories.length, 15);
    await _bluebird.default.map(factories, async factory => {
      t.truthy(factory.id);
      const node = await tree.getById(factory.id);
      t.truthy(node.id.match(UUIDV4REGEX));
      t.true(Array.isArray(node.numbers));
      t.true((0, _isString2.default)(node.name) && !(0, _isEmpty2.default)(node.name));
      t.deepEqual((0, _omit2.default)(node, ['createdAt', 'updatedAt', 'name']), {
        id: node.id,
        numbers: [],
        upperGenerateLimit: 15,
        lowerGenerateLimit: 1,
        deletedAt: null
      });
    });
  });
  (0, _ava.default)('Factories can generate random numbers', async t => {
    t.plan(30);
    const factories = await tree.generateFactory(15);
    await _bluebird.default.map(factories, async factory => {
      await tree.generateNumber(factory, 10);
      const updatedFactory = await tree.getById(factory.id);
      t.is(updatedFactory.numbers.length, 10);
      t.true(updatedFactory.numbers[0] > 0 && updatedFactory.numbers[0] < _maxSafeInteger.default);
    });
  });
  (0, _ava.default)('When generating more than 15 numbers an error is thrown.', async t => {
    const [factory] = await tree.generateFactory(1);
    const promise = tree.generateNumber(factory, 16);
    await t.throws(promise, RangeError, 'The operation failed because the amount of nodes would be greater than 15.');
  });
  (0, _ava.default)('When generating less than 1 number an error is thrown.', async t => {
    const [factory] = await tree.generateFactory(1);
    const updatedFactory = await tree.generateNumber(factory, 0);
    t.is(updatedFactory.numbers.length, 0);
  });
  (0, _ava.default)('Factories should have an adjustable name.', async t => {
    const factories = await tree.generateFactory(1);
    const factory = await tree.getById(factories[0].id);
    await tree.setName(factory, 'test');
    const updatedFactory = await tree.getById(factories[0].id);
    t.is(updatedFactory.name, 'test');
  });

  _ava.default.skip('Factories should have an adjustable lower number bound.', async t => {
    const factories = await tree.generateFactory(1);
    const factory = await tree.getById(factories[0].id);
    await tree.setLowerGenerateLimit(factory, 5);
    const updatedFactory = await tree.getById(factories[0].id);
    t.is(updatedFactory.lowerGenerateLimit, 5);
    const promise = tree.generateNumber(factory, 4);
    await t.throws(promise, RangeError, 'The operation failed because the amount of nodes is less than 4');
  });

  _ava.default.skip('Factories should have an adjustable upper number bound.', async t => {
    const factories = await tree.generateFactory(1);
    const factory = await tree.getById(factories[0].id);
    await tree.setUpperGenerateLimit(factory, 10);
    const updatedFactory = await tree.getById(factories[0].id);
    t.is(updatedFactory.upperGenerateLimit, 10);
    const promise = tree.generateNumber(factory, 11);
    await t.throws(promise, RangeError, 'The operation failed because the amount of nodes would be greater than 10.');
  });

  (0, _ava.default)('A callback is called for every set operation', async t => {
    t.plan(6);
    let count = 0;

    const callback = key => {
      t.true((0, _isFunction2.default)(tree[key]));
      count += 1;
    };

    const callbackTree = (0, _tree.default)((await store), callback);
    const [factory] = await callbackTree.generateFactory(1);
    await callbackTree.generateNumber(factory, 10);
    await callbackTree.setName(factory, 'hey');
    await callbackTree.setLowerGenerateLimit(factory, 10);
    await callbackTree.setUpperGenerateLimit(factory, 100);
    t.is(count, 5);
  });
  /**
   * This test is serial because it needs to count that
   * ONLY one was removed. This is very important so we
   * Need to not let other ops happen during the test.
   */

  _ava.default.serial('You can "remove" a factory by setting deleted to true.', async t => {
    await tree.generateFactory(15);
    const originalFactories = await tree.getFactories();
    const toDelete = originalFactories[0];
    await tree.deleteFactory(toDelete);
    const newFactories = await tree.getFactories();
    t.is(newFactories.length, 14);
    const deleted = await tree.getById(toDelete.id);
    t.deepEqual((0, _omit2.default)(deleted, 'deletedAt'), (0, _objectSpread2.default)({}, (0, _omit2.default)(toDelete, 'deletedAt')));
    t.true(typeof deleted.deletedAt === 'string');
  });

  (0, _ava.default)('Deleting a factory returns the deleted factory', async t => {
    const [factory] = await tree.getFactories();
    const deleted = await tree.deleteFactory(factory);
    t.deepEqual((0, _omit2.default)(factory, 'deletedAt'), (0, _omit2.default)(deleted, 'deletedAt'));
    t.true(typeof deleted.deletedAt === 'string');
  });
  (0, _ava.default)('All setter operations return the new factory.', async t => {
    const factories = await tree.getFactories();
    const factory = (0, _sample2.default)(factories);
    const withNewName = await tree.setName(factory, 'unique');
    t.deepEqual(withNewName, (0, _objectSpread2.default)({}, factory, {
      name: 'unique'
    }));
  });
};

exports.default = _default;
//# sourceMappingURL=abstractTests.js.map