import _ from 'lodash';
import uuid from 'uuid/v4';
import Promise from 'bluebird';
import Sequelize from 'sequelize';

/**
 * This memory store is meant to be a stand in for the database in unit tests.
 * It uses the same interface but instead interacts to a Map to reduce IO.
 */
export default async (options) => {
  const sequelize = new Sequelize(options, {
    logging: false,
  });

  const node = sequelize.define('node', {
    id: {
      type: Sequelize.STRING,
      unique: true,
      allowNull: false,
      defaultValue: uuid,
      primaryKey: true,
    },
    numbers: {
      type: Sequelize.STRING,
    },
    name: {
      type: Sequelize.STRING,
    },
    upperGenerateLimit: {
      type: Sequelize.INTEGER,
      allowNull: false,
    },
    lowerGenerateLimit: {
      type: Sequelize.INTEGER,
      allowNull: false,
    },
  }, {
    timestamps: true,
    paranoid: true,
  });

  await node.sync();

  const methods = {
    getFactories() {
      return Promise.map(node.findAll({
        raw: true,
      }), (res) => {
        const numbers = _.get(res, 'numbers');

        Object.assign(res, {
          numbers: JSON.parse(numbers),
        });

        return res;
      });
    },
    /**
     * @param {UUID} id This ID will be compared to node.id
     * If there is a match then the node value will be deleted.
     */
    async deleteNodeById(id) {
      await node.destroy({
        where: {
          id,
        },
        limit: 1,
      });

      return methods.getNodeById(id);
    },
    /**
     * @param {UUID} id This ID will be compared to node.id
     * If there is a match then the node value will be returned.
     */
    async getNodeById(id) {
      const res = await node.findOne({
        where: {
          id,
        },
        raw: true,
        paranoid: false,
      });

      const numbers = _.get(res, 'numbers');

      Object.assign(res, {
        numbers: JSON.parse(numbers),
      });

      return res;
    },
    /**
     * @param {UUID} id the node key to upsert.
     * @param {Object} value The nodes value to upsert.
     */
    async upsertById(id, val) {
      Object.assign(val, {
        numbers: JSON.stringify(_.castArray(val.numbers)),
      });

      await node.upsert(val, {
        where: {
          id,
        },
      });

      return methods.getNodeById(id);
    },
  };

  return methods;
};
