import _ from 'lodash';
import faker from 'faker';
import fp from 'lodash/fp';
import uuid from 'uuid/v4';
import Promise from 'bluebird';

import {
  maxNodesExceededError,
} from './treeErrors';

function createNode({ id = uuid(), ...defaults } = {}) {
  return {
    id,
    factories: [],
    numbers: [],
    parent: null,
    name: faker.name.findName(),
    upperGenerateLimit: 15,
    lowerGenerateLimit: 1,
    ...defaults,
  };
}

const createFactory = fp.compose(
  fp.assign({ numbers: [] }),
  fp.omit('factories'),
  createNode,
);

export default function tree(store, callback = _.noop) {
  const getters = {
    getById: store.getNodeById,
    getFactories: store.getFactories,
  };

  const setters = {
    async generateFactory(number) {
      const factories = [];

      _.times(number, () => factories.push(createFactory()));

      return Promise.map(factories, factory => store.upsertById(factory.id, factory));
    },
    async generateNumber(node, number) {
      const storedNode = await store.getNodeById(node.id);

      if (number > 15) {
        return maxNodesExceededError(15);
      }

      const numbers = [];

      _.times(
        number,
        () => numbers.push(_.random(
          storedNode.lowerGenerateLimit,
          storedNode.upperGenerateLimit,
        )),
      );

      return store.upsertById(storedNode.id, {
        ...storedNode,
        numbers,
      });
    },
    async setName(node, name) {
      const storedNode = await store.getNodeById(node.id);

      return store.upsertById(storedNode.id, {
        ...storedNode,
        name,
      });
    },
    async setLowerGenerateLimit(node, lowerGenerateLimit) {
      const storedNode = await store.getNodeById(node.id);

      return store.upsertById(storedNode.id, {
        ...storedNode,
        lowerGenerateLimit,
      });
    },
    async setUpperGenerateLimit(node, upperGenerateLimit) {
      const storedNode = await store.getNodeById(node.id);

      return store.upsertById(storedNode.id, {
        ...storedNode,
        upperGenerateLimit,
      });
    },
    deleteFactory(node) {
      return store.deleteNodeById(node.id);
    },
  };

  const callbackSetters = _.mapValues(setters, (fn, key) => async (...args) => {
    const res = await fn(...args);

    callback(key, res, ...args);

    return res;
  });

  return Object.assign({}, getters, callbackSetters);
}
